//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.Activities
{
    #region Using Directives

    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.ComponentModel;
    using System.Workflow.ComponentModel;
    using System.Collections;
    using System.Collections.Specialized;
    using System.Workflow.ComponentModel.Design;
    using System.ComponentModel.Design;
    using Microsoft.Research.ScientificWorkflow.NetCDF.CSharpAPI;
    using System.Workflow.ComponentModel.Compiler;

    using Microsoft.Research.ScientificWorkflow;
    using Microsoft.Research.ScientificWorkflow.Helpers;
    
    using WorkflowData;
    #endregion

    #region MeanTransformerActivity Class

    [Name("Variable Mean Transform")]
    [Description("Transform for finding the mean values for a variable")]
    [WorkflowCategory("NetCdf")]
    public class MeanTransformerActivity : Activity
    {
        #region Private Fields

        private const string HyperCuberPropertyName = "GenerateHyperCubeActivity";
        private const string MissingValue = "missing_value";

        #endregion

        #region Dependency Properties

        public static DependencyProperty VariableNameProperty = DependencyProperty.Register("VariableName",
            typeof(string), typeof(MeanTransformerActivity));

        [RequiredInputParam]
        [Name("Variable Name")]
        [Description("Variable Name for which the min/max value is to be found.")]
        public string VariableName
        {
            get
            {
                return ((string)(base.GetValue(MeanTransformerActivity.VariableNameProperty)));
            }
            set
            {
                base.SetValue(MeanTransformerActivity.VariableNameProperty, value);
            }
        }

        public static DependencyProperty InputCubeProperty =
            DependencyProperty.Register("InputCube", typeof(HyperCube), typeof(MeanTransformerActivity));

        [RequiredInputParam]
        [Name("In Hyper Cube")]
        [Description("Hyper Cube for finding Mean")]
        public HyperCube InputCube
        {
            get
            {
                return ((HyperCube)(base.GetValue(MeanTransformerActivity.InputCubeProperty)));
            }
            set
            {
                base.SetValue(MeanTransformerActivity.InputCubeProperty, value);
            }
        }

        public static DependencyProperty MeanProperty = DependencyProperty.Register("Mean",
            typeof(object), typeof(MeanTransformerActivity));

        /// <summary>
        /// Mean Output property
        /// </summary>
        [OutputParam]
        [Name("Mean Value")]
        [Description("Lowest value for the specified variable.")]
        public object Mean
        {
            get
            {
                return ((object)(base.GetValue(MeanTransformerActivity.MeanProperty)));
            }
            set
            {
                base.SetValue(MeanTransformerActivity.MeanProperty, value);
            }
        }

        #endregion

        #region Overridden Methods

        /// <summary>
        /// Extract mean value
        /// </summary>
        /// <param name="executionContext"></param>
        /// <returns></returns>
        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            //Get the HyperCube form earlier activity and proceed for min and max
            HyperCube InputHyperCube = this.InputCube;

            int i = InputHyperCube.Schema.LookupColumnIndex(this.VariableName);
            if (-1 == i)
                throw new ArgumentException("MinMaxTransform:Invalid Variable Name");
            Type typ = InputHyperCube.Schema.GetColumnType(i);

            object missingValue = InputHyperCube.Schema.GetAttribute(i, ScalerTransform.MissingValue);

            int xLen = InputHyperCube.Schema.HasDimension(i, HyperCube.Axis.X) ? 
                InputHyperCube.GetAxisLength(HyperCube.Axis.X) : 1;
            int yLen = InputHyperCube.Schema.HasDimension(i, HyperCube.Axis.Y) ? 
                InputHyperCube.GetAxisLength(HyperCube.Axis.Y) : 1;
            int zLen = InputHyperCube.Schema.HasDimension(i, HyperCube.Axis.Z) ? 
                InputHyperCube.GetAxisLength(HyperCube.Axis.Z) : 1;
            int tLen = InputHyperCube.Schema.HasDimension(i, HyperCube.Axis.T) ? 
                InputHyperCube.GetAxisLength(HyperCube.Axis.T) : 1;

            // Since the sum could be huge value use double
            double mean = 0;

            for (int x = 0; x < xLen; x++)
            {
                for (int y = 0; y < yLen; y++)
                {
                    for (int z = 0; z < zLen; z++)
                    {
                        for (int t = 0; t < tLen; t++)
                        {
                            object val = InputHyperCube[x, y, z, t][i];
                            if (val is byte)
                            {
                                byte misVal = ((byte[])missingValue)[0];
                                bool IsMissing = (null != missingValue) ? Convert.ToByte(val) == misVal : false;
                                mean += (true == IsMissing) ? 0 : Convert.ToByte(val);
                            }
                            else if (val is short)
                            {
                                short misVal = ((short[])missingValue)[0];
                                bool IsMissing = (null != missingValue) ? Convert.ToInt16(val) == misVal : false;
                                mean += (true == IsMissing) ? 0 : Convert.ToInt16(val);
                            }
                            else if (val is int)
                            {
                                int misVal = ((int[])missingValue)[0];
                                bool IsMissing = (null != missingValue) ? Convert.ToInt32(val) == misVal : false;
                                mean += (true == IsMissing) ? 0 : Convert.ToInt32(val);
                            }
                            else if (val is float)
                            {
                                float misVal = ((float[])missingValue)[0];
                                bool IsMissing = (null != missingValue) ? Convert.ToSingle(val) == misVal : false;
                                mean += (true == IsMissing) ? 0 : Convert.ToSingle(val);
                            }
                            else if (val is double)
                            {
                                double misVal = ((double[])missingValue)[0];
                                bool IsMissing = (null != missingValue) ? Convert.ToDouble(val) == misVal : false;
                                mean += (true == IsMissing) ? 0 : Convert.ToDouble(val);
                            }
                        }
                    }
                }
            }
            this.Mean = mean / (double)(xLen * yLen * zLen * tLen);
            return ActivityExecutionStatus.Closed;
        }

        #endregion

    }

    #endregion
}