//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.DataLayer.ServiceModel
{
    using System;
    using System.Globalization;

    /// <summary>
    /// A model describing the meta-data associated with a workflow parameter
    /// of a workflow activity. This data includes whether it is an input or
    /// output paramter, as well as the name, label, and description attached
    /// to that parameter.
    /// </summary>
    [Serializable]
    public class ParameterDescriptionModel
    {
        #region private variables

        /// <summary>
        /// The Unique id for the parameter.
        /// </summary>
        private Guid id = Guid.Empty;

        /// <summary>
        /// Name of the property as in the custom attribute.
        /// </summary>
        private string activityName;

        /// <summary>
        /// Name of the property as in the class.
        /// </summary>
        private string propertyName;

        /// <summary>
        ///  Description of the parameter.
        /// </summary>
        private string description;

        /// <summary>
        /// Datatype of the parameter.
        /// </summary>
        private string dataType;

        /// <summary>
        /// Value of the parameter.
        /// </summary>
        private string value;

        /// <summary>
        /// A flag which specifies if the parameter is mandatory for the workflow. 
        /// </summary>
        private bool isMandatoryForWorkflow;

        /// <summary>
        /// A flag which specifies whether the parameter is databound to another parameter.
        /// </summary>
        private bool isDatabound;

        /// <summary>
        /// Label of the description model
        /// </summary>
        private string label;

        /// <summary>
        /// A flag which indicates whether the value is valid.
        /// </summary>
        private bool? isValueInvalid;

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="source">Activity parameter object.</param>
        public ParameterDescriptionModel(ParameterAssignment source)
        {
            #region Validation of Arguments
            if (null == source)
            {
                throw new ArgumentNullException("source");
            }
            #endregion

            if (source != null)
            {
                // Set the property Name of the Parameter.
                this.propertyName = source.ActivityParameter.ParameterClass;

                // Set the Name of the Parameter.
                this.activityName = source.ActivityParameter.Activity.Name;

                // Set the user friendly Name of the Parameter.
                this.label = source.ActivityParameter.Label;

                // Set the description of the Parameter.
                this.description = source.ActivityParameter.Description;

                this.isDatabound = source.IsBound;
                if (source.IsBound)
                {

                    this.Value = string.Format("{0}.{1}", source.BoundTo.ActivityParameter.Activity.Name, source.BoundToPath);

                }
                else
                {
                    this.Value = source.CurrentValue;
                }

                this.isMandatoryForWorkflow = source.RequiredForWorkflow;

                // Set datatype.
                this.dataType = source.ActivityParameter.Type;

                this.AutomationId = string.Format("{0}_{1}", source.ActivityParameter.Activity.Name, this.activityName);// automationIdBuilder.ToString();
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ParameterDescriptionModel"/> class.
        /// </summary>
        public ParameterDescriptionModel()
        {
        }

        #endregion

        #region public properties

        /// <summary>
        /// The Unique id for the parameter.
        /// </summary>
        public Guid Id
        {
            get
            {
                return id;
            }
        }

        /// <summary>
        /// Name of the parameter as in the class.
        /// </summary>
        public string PropertyName
        {
            get
            {
                return propertyName;
            }
        }

        /// <summary>
        /// Name of the parameter as in attribute.
        /// </summary>
        public string ActivityName
        {
            get
            {
                return this.activityName;
            }
        }

        /// <summary>
        /// Name of the parameter.
        /// </summary>
        public string Label
        {
            get
            {
                return this.label;
            }
        }

        /// <summary>
        /// Description of the parameter.
        /// </summary>
        public string Description
        {
            get
            {
                return description;
            }
        }

        /// <summary>
        /// Datatype of the parameter.
        /// </summary>
        public string DataType
        {
            get
            {
                return dataType;
            }
        }

        /// <summary>
        /// Value of the parameter.
        /// </summary>
        public string Value
        {
            get
            {
                return value;
            }
            set
            {
                if (value != this.value)
                {
                    this.value = value;
                }
            }
        }

        /// <summary>
        /// Gets or Sets a flag which specifies if the parameter is mandatory for the workflow execution. 
        /// </summary>
        public bool IsMandatoryForWorkflow
        {
            get
            {
                return this.isMandatoryForWorkflow;
            }
            set
            {
                this.isMandatoryForWorkflow = value;
            }
        }

        /// <summary>
        /// A flag which signifies whether the parameter is mandatory for the workflow execution.
        /// </summary>
        public bool IsNotMandatoryForWorkflow
        {
            get
            {
                return !this.isMandatoryForWorkflow;
            }
        }

        /// <summary>
        /// A flag which specifies whether the parameter is databound to another parameter.
        /// </summary>
        public bool IsDatabound
        {
            get
            {
                return isDatabound;
            }
            set
            {
                if (value != this.isDatabound)
                {
                    this.isDatabound = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the automation id.
        /// </summary>
        /// <value>The automation id.</value>
        public string AutomationId
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the flag which indicates whether the value is valid. To be accessed only after it is loaded in the property pane.
        /// Will not hold a proper value before loading.
        /// </summary>
        public bool? IsValueInvalid
        {
            get
            {
                return this.isValueInvalid;
            }
            set
            {
                this.isValueInvalid = value;
            }
        }

        /// <summary>
        /// Gets or Sets the name of the activity in which this activity is present. Used only for naming in the property list.
        /// </summary>
        public string PropertyPaneDisplayName
        {
            get
            {
                return string.Format("{0}.{1}", this.activityName, this.propertyName);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public string ActivityUniqueId { get; set; }

        #endregion

        #region Public Methods

        /// <summary>
        /// Validate the entered value against the requirements of the paramModel.
        /// </summary>
        /// <param name="enteredVal">
        /// The current value.
        /// </param>
        /// <param name="paramDesc">
        /// The parameter model for which the value has been specified.
        /// </param>
        /// <returns>
        /// Returns true if valid value, false otherwise.
        /// </returns>
        public static bool ValidateInputValue(string enteredVal, ParameterDescriptionModel paramDesc)
        {
            bool validValue = true;
            if (paramDesc != null)
            {
                if (string.IsNullOrEmpty(enteredVal))
                {
                    return false;
                }
                else
                {
                    try
                    {
                        System.Type dataType = System.Type.GetType(paramDesc.DataType);
                        if (dataType != null)
                        {
                            // System.ComponentModel.TypeConverter converter = new TypeConverter();
                            object parsedValue = System.Convert.ChangeType(enteredVal, dataType, CultureInfo.CurrentCulture);
                            // object parsedValue = converter.ConvertTo(enteredVal, dataType);
                            if (null == parsedValue)
                            {
                                validValue = false;
                            }
                        }
                    }
                    catch (NotSupportedException)
                    {
                        validValue = false;
                    }
                    catch (FormatException)
                    {
                        validValue = false;
                    }
                    catch (Exception)
                    {
                        validValue = false;
                    }
                }
            }

            return validValue;
        }

        /// <summary>
        /// The string representation of the object. Return the display name. Used in automation.
        /// </summary>
        /// <returns>The string representation of the object.</returns>
        public override string ToString()
        {
            if (!string.IsNullOrEmpty(this.PropertyPaneDisplayName))
            {
                return this.PropertyPaneDisplayName;
            }
            return base.ToString();
        }

        #endregion
    }
}
