//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.ManagementStudioModels
{
    using System;
    using System.Windows;
    using SR = Microsoft.Research.DataLayer;
    using System.Collections.ObjectModel;
    using System.Linq;
    using System.Collections;

    /// <summary>
    /// A view friendly model for the User.
    /// </summary>
    public class UserViewModel : BaseRegMgrModel
    {
        #region Private fields

        private Guid id;

        private string centerPaneHeader;

        private bool nonVirtualUser;

        private bool status;

        private Guid displayId;

        private DateTime displayCreatedDate;

        private DateTime displayLastModifiedTime;

        private bool isDeleted;
        
        #endregion

        #region Constructors

        /// <summary>
        /// Constructor. An empty object will be created.
        /// </summary>
        public UserViewModel()
        {
            this.IsSelected = false;
            this.Status = true;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserViewModel"/> class. Initialize the model with information from the 
        /// passed category.
        /// </summary>
        /// <param name="user">The namespace item.</param>
        public UserViewModel(SR.User user, UserViewModel parent)
        {
            this.Name = user.Name;
            this.Label = user.Name;
            this.Id = (user as SR.IObject).ID;
            this.Description = user.Description;
            this.CreatedBy = user.Created.User;
            this.CreatedDate = user.Created.When;
            this.LastModifiedBy = user.Updated.User;
            this.LastModifiedTime = user.Updated.When;
            this.Parent = parent;
            this.NonVirtualUser = true;
            this.Status = user.Enabled;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UserViewModel"/> class.
        /// Initialize from the passed model.
        /// </summary>
        /// <param name="model">The model.</param>
        private UserViewModel(UserViewModel model)
        {
            if (model != null)
            {
                this.Name = model.Name;
                this.Label = model.Label;
                this.Id = model.id;
                this.Description = model.Description;
                this.CreatedBy = model.CreatedBy;
                this.CreatedDate = model.CreatedDate;
                this.LastModifiedBy = model.LastModifiedBy;
                this.LastModifiedTime = model.LastModifiedTime;
                this.IsSelected = model.IsSelected;
                this.Parent = model.Parent;
                this.NonVirtualUser = model.NonVirtualUser;
                this.Status = model.Status;
                if (model.Credentials != null)
                {
                    this.CloneCredentialCollection(model.Credentials);
                }
                if (model.Roles != null)
                {
                    this.CloneRoleCollection(model.Roles);
                }
                if (model.Groups != null)
                {
                    this.CloneGroupCollection(model.Groups);
                }
            }
        }


        private void CloneCredentialCollection(ObservableCollection<LogOnCredentialModel> loginModels)
        {
            this.Credentials = new ObservableCollection<LogOnCredentialModel>();

            foreach (LogOnCredentialModel model in loginModels)
            {
                LogOnCredentialModel cloneModel = new LogOnCredentialModel(model.CredentialName, model.CredentialDisplayName);
                this.Credentials.Add(cloneModel);
            }
        }

        private void CloneRoleCollection(ObservableCollection<CredentialRoleModel> rolesModels)
        {
            this.Roles = new ObservableCollection<CredentialRoleModel>();

            foreach (CredentialRoleModel model in rolesModels)
            {
                CredentialRoleModel cloneModel = new CredentialRoleModel(model.RoleName, model.RoleSelected);
                this.Roles.Add(cloneModel);
            }
        }

        private void CloneGroupCollection(ObservableCollection<CredentialGroupModel> groupModels)
        {
            this.Groups = new ObservableCollection<CredentialGroupModel>();

            foreach (CredentialGroupModel model in groupModels)
            {
                CredentialGroupModel cloneModel = new CredentialGroupModel(model.GroupName, model.IsSelected);
                this.Groups.Add(cloneModel);
            }
        }

        #endregion Constructors

        #region Public Properties

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id.</value>
        public override Guid Id
        {
            get
            {
                return id;
            }
            set
            {
                id = value;
                this.DisplayId = value;
                this.RaisePropertyChanged("Id");
            }
        }

        /// <summary>
        /// Gets or sets the created date.
        /// </summary>
        /// <value>The created date.</value>
        public override DateTime CreatedDate
        {
            get
            {
                return base.CreatedDate;
            }
            set
            {
                base.CreatedDate = value;
                this.DisplayCreatedDate = value;
            }
        }

        /// <summary>
        /// Gets or sets the last modified time.
        /// </summary>
        /// <value>The last modified time.</value>
        public override DateTime LastModifiedTime
        {
            get
            {
                return base.LastModifiedTime;
            }
            set
            {
                base.LastModifiedTime = value;
                this.DisplayLastModifiedTime = value;
            }
        }

        /// <summary>
        /// Gets or sets the display id.
        /// </summary>
        /// <value>The display id.</value>
        public Guid DisplayId
        {
            get
            {
                if (this.displayId.Equals(Guid.Empty))
                {
                    return (Guid)DependencyProperty.UnsetValue;
                }

                return this.displayId;
            }
            set
            {
                this.displayId = value;
                this.RaisePropertyChanged("DisplayId");
            }
        }

        /// <summary>
        /// Gets or sets the display created date.
        /// </summary>
        /// <value>The display created date.</value>
        public DateTime DisplayCreatedDate
        {
            get
            {
                if (DateTime.MinValue.Equals(this.displayCreatedDate))
                {
                    return (DateTime)DependencyProperty.UnsetValue;
                }

                return this.displayCreatedDate;
            }
            set
            {
                this.displayCreatedDate = value;
                this.RaisePropertyChanged("DisplayCreatedDate");
            }
        }

        /// <summary>
        /// Gets or sets the display last modified time.
        /// </summary>
        /// <value>The display last modified time.</value>
        public DateTime DisplayLastModifiedTime
        {
            get
            {
                if (DateTime.MinValue.Equals(this.displayLastModifiedTime))
                {
                    return (DateTime)DependencyProperty.UnsetValue;
                }

                return this.displayLastModifiedTime;
            }
            set
            {
                this.displayLastModifiedTime = value;
                this.RaisePropertyChanged("DisplayLastModifiedTime");
            }
        }

        /// <summary>
        /// Gets or sets the center pane header.
        /// </summary>
        public string CenterPaneHeader
        {
            get
            {
                return this.centerPaneHeader;
            }
            set
            {
                this.centerPaneHeader = value;
                this.RaisePropertyChanged("CenterPaneHeader");
            }
        }

        /// <summary>
        /// Gets or set the Non Virtual user
        /// </summary>
        public bool NonVirtualUser
        {
            get
            {
                return this.nonVirtualUser;
            }
            set
            {
                this.nonVirtualUser = value;
                this.RaisePropertyChanged("NonVirtualUser");
            }
        }

        /// <summary>
        /// Gets or sets the Status.
        /// </summary>
        /// <value>The Status.</value>
        public bool Status
        {
            get
            {
                return status;
            }
            set
            {
                this.status = value;
                this.RaisePropertyChanged("Status");
            }
        }

        /// <summary>
        /// get and  private set Credentials
        /// </summary>
        public ObservableCollection<LogOnCredentialModel> Credentials { get; set; }

        /// <summary>
        /// get and  private set Roles
        /// </summary>
        public ObservableCollection<CredentialRoleModel> Roles { get; set; }

        /// <summary>
        /// get and  private set Groups
        /// </summary>
        public ObservableCollection<CredentialGroupModel> Groups { get; set; }

        /// <summary>
        ///  A flag indication if the workflow is deleted.
        /// </summary>
        public bool IsDeleted
        {
            get
            {
                return isDeleted;
            }
            set
            {
                isDeleted = value;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Creates the clone.
        /// </summary>
        /// <param name="model">The model.</param>
        /// <returns>
        /// The cloned model.
        /// </returns>
        public static UserViewModel CreateClone(UserViewModel model)
        {
            UserViewModel clonedModel = null;
            if (null != model)
            {
                clonedModel = new UserViewModel(model);
            }

            return clonedModel;
        }



        /// <summary>
        /// Serves as a hash function for a particular type.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"/>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }


        public static bool CheckGroupCollectionEquality(ObservableCollection<CredentialGroupModel> obj1, ObservableCollection<CredentialGroupModel> obj2)
        {

            if ((obj1 == null && obj2 == null))
            {
                return true;
            }

            if (obj1 != null && obj2 != null && obj1.Count != obj2.Count)
            {
                return false;
            }

            foreach (CredentialGroupModel model in obj1)
            {
                CredentialGroupModel resultModel = obj2.FirstOrDefault(p => p.GroupName.Equals(model.GroupName) && p.IsSelected != model.IsSelected);
                if (resultModel != null)
                {
                    return false;
                }

            }

            return true;
        }


        /// <summary>
        /// Checks the equality of Roles Collection
        /// </summary>
        /// <param name="obj1">The obj1.</param>
        /// <param name="obj2">The obj2.</param>
        /// <returns></returns>
        public static bool CheckRolesCollectionEquality(ObservableCollection<CredentialRoleModel> obj1, ObservableCollection<CredentialRoleModel> obj2)
        {

            if ((obj1 == null && obj2 == null))
            {
                return true;
            }

            if (obj1 != null && obj2 != null && obj1.Count != obj2.Count)
            {
                return false;
            }

            foreach (CredentialRoleModel model in obj1)
            {
                CredentialRoleModel resultModel = obj2.FirstOrDefault(p => p.RoleName.Equals(model.RoleName) && p.RoleSelected != model.RoleSelected);
                if (resultModel != null)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Checks the equality of Roles Collection
        /// </summary>
        /// <param name="obj1">The obj1.</param>
        /// <param name="obj2">The obj2.</param>
        /// <returns></returns>
        public static bool CheckCredentialCollectionEquality(ObservableCollection<LogOnCredentialModel> obj1, ObservableCollection<LogOnCredentialModel> obj2)
        {
            if ((obj1 == null && obj2 == null))
            {
                return true;
            }

            if (obj1 != null && obj2 != null && obj1.Count != obj2.Count)
            {
                return false;
            }

            foreach (LogOnCredentialModel model in obj1)
            {
                LogOnCredentialModel resultModel = obj2.FirstOrDefault(p => p.CredentialName.Equals(model.CredentialName));
                if (resultModel == null)
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"/> to compare with the current <see cref="T:System.Object"/>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"/> is equal to the current <see cref="T:System.Object"/>; otherwise, false.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">
        /// The <paramref name="obj"/> parameter is null.
        /// </exception>
        public override bool Equals(object obj)
        {
            bool equal = false;
            UserViewModel equateParam = obj as UserViewModel;
            if (equateParam != null &&
                equateParam.Id.Equals(this.Id) &&
                UserViewModel.CheckEquality(equateParam.Description, this.Description) &&
                UserViewModel.CheckEquality(equateParam.Name, this.Name) &&
                 UserViewModel.CheckEquality(equateParam.Status, this.Status) &&
                 UserViewModel.CheckEquality(equateParam.Description, this.Description) &&
                 UserViewModel.CheckGroupCollectionEquality(equateParam.Groups, this.Groups) &&
                 UserViewModel.CheckCredentialCollectionEquality(equateParam.Credentials, this.Credentials) &&
                 UserViewModel.CheckRolesCollectionEquality(equateParam.Roles, this.Roles))
            {
                equal = true;
            }

            return equal;
        }


        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return this.Name;
        }

        /// <summary>
        /// Updates the specified model.
        /// </summary>
        /// <param name="model">The model.</param>
        public override void Update(BaseRegMgrModel model)
        {
            UserViewModel userModel = model as UserViewModel;
            if (userModel != null)
            {
                this.Name = userModel.Name;
                this.Label = userModel.Label;
                this.Description = userModel.Description;
                this.CreatedDate = userModel.CreatedDate;
                this.CreatedBy = userModel.CreatedBy;
                this.LastModifiedTime = userModel.LastModifiedTime;
                this.LastModifiedBy = userModel.LastModifiedBy;
                this.IsSelected = userModel.IsSelected;
                this.Parent = model.Parent;
                this.Status = userModel.Status;
                this.Roles = userModel.Roles;
                this.Credentials = userModel.Credentials;
                this.Groups = userModel.Groups;
            }
        }

        /// <summary>
        /// Updates the specified user.
        /// </summary>
        /// <param name="category">The category.</param>
        public void Update(SR.User user)
        {
            this.Name = user.Name;
            this.Label = string.IsNullOrEmpty(user.Name) ? user.Name : string.Empty;
            this.Id = (user as SR.IObject).ID;
            this.Description = user.Description;
            this.CreatedBy = user.Created.User;
            this.CreatedDate = user.Created.When;
            this.LastModifiedBy = user.Updated.User;
            this.LastModifiedTime = user.Updated.When;
            this.Status = user.Enabled;
            this.NonVirtualUser = true;
        }

        #endregion Public Methods
    }


}
