//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.Provenance.Controls.Presenter
{
    using System.Collections.ObjectModel;
    using System.Windows;
    using Microsoft.Research.Provenance.Controls.View;
    using Microsoft.Research.Provenance.ProvenanceDataService;
    using Microsoft.Research.ProvenanceCommon.Model;
    using Microsoft.Research.ProvenanceInterfaces;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    using System.ComponentModel;
    using Microsoft.Practices.Unity;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
    using System.Text;

    public class ActivityProcessingStatusPresenter : IProvenancePresenter, INotifyPropertyChanged
    {
        /// <summary>
        /// The helper class which fetches data from the registry.
        /// </summary>
        // private DataRetriever dataRetriever;

        /// <summary>
        /// Holds the UnityContainer
        /// </summary>
        private IUnityContainer container;


        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="view">The view associated with the presenter.</param>
        public ActivityProcessingStatusPresenter(IUnityContainer container, ActivityProcessingStatusView view)
        {
            this.View = view;
            this.container = container;

            this.ActivityProcessingStatusModels = new ObservableCollection<ProcessingStatusModel>();
            view.Model = this;
        }

        /// <summary>
        /// The collection of processing status belonging to an activity.
        /// </summary>
        public ObservableCollection<ProcessingStatusModel> ActivityProcessingStatusModels { get; private set; }

        /// <summary>
        /// The control which displays the processing status.
        /// </summary>
        public UIElement View { get; set; }

        /// <summary>
        /// Activity label.
        /// </summary>
        private string activityLabel;

        /// <summary>
        /// Activity label.
        /// </summary>
        public string ActivityLabel
        {
            get
            {
                return this.activityLabel;
            }
            private set
            {
                this.activityLabel = value;
                this.OnNotifyPropertyChanged("ActivityLabel");
            }
        }

        /// <summary>
        /// Inidicates whether data is not present.
        /// </summary>
        private bool doesNotContainItems;

        /// <summary>
        /// Gets whether data is not present.
        /// </summary>
        public bool DoesNotContainItems
        {
            get
            {
                return this.doesNotContainItems;
            }
            private set
            {
                this.doesNotContainItems = value;
                this.OnNotifyPropertyChanged("DoesNotContainItems");
            }
        }

        /// <summary>
        /// Inidicates whether data is present.
        /// </summary>
        private bool containItems;

        /// <summary>
        /// Gets whether data is present.
        /// </summary>
        public bool ContainItems
        {
            get
            {
                return this.containItems;
            }
            private set
            {
                this.containItems = value;
                this.DoesNotContainItems = !this.ContainItems;
                this.OnNotifyPropertyChanged("ContainItems");
            }
        }

        /// <summary>
        /// BreadCrumb text binded with UI.
        /// </summary>
        private StringBuilder breadcrumb;

        /// <summary>
        /// Gets or sets the Breadcrumb property, binded with UI.
        /// </summary>
        public string Breadcrumb
        {
            get
            {
                return this.breadcrumb.ToString();
            }
            set
            {
                this.breadcrumb = new StringBuilder();
                this.breadcrumb.Append(value);

                // Don't display breadcrumb if not applicable.
                if (string.IsNullOrEmpty(this.Breadcrumb))
                    IsBreadcrumbNull = false;
                else
                    IsBreadcrumbNull = true;

                this.OnNotifyPropertyChanged("Breadcrumb");
            }
        }

        /// <summary>
        /// Inidicates whether breadcrumb text is not present.
        /// </summary>
        private bool isBreadcrumbNull;

        /// <summary>
        /// Gets or sets a value indicating whether breadcrumb is null.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if breadcrumb is null; otherwise, <c>false</c>.
        /// </value>
        public bool IsBreadcrumbNull
        {
            get
            {
                return this.isBreadcrumbNull;
            }
            private set
            {
                this.isBreadcrumbNull = value;
                this.OnNotifyPropertyChanged("IsBreadcrumbNull");
            }
        }

        #region IProvenancePresenter Members

        /// <summary>
        /// Hide the view.
        /// </summary>
        public void Hide()
        {
            this.ActivityProcessingStatusModels.Clear();
            this.View.Visibility = Visibility.Collapsed;
        }

        /// <summary>
        /// Show the view and retrieve the processing status.
        /// </summary>
        /// <param name="compositeModel">The activity name.</param>
        public void Show(BaseModel model)
        {
            string breadcrumb = string.Empty;

            try
            {
                this.ActivityProcessingStatusModels.Clear();

                this.ActivityLabel = model.Name;

                Collection<ProcessingStatusModel> modelsFromRegistry = this.container.Resolve<DataRetriever>().GetProcessingStatus(model, ref breadcrumb);
                if (modelsFromRegistry != null)
                {
                    foreach (ProcessingStatusModel processingModel in modelsFromRegistry)
                    {
                        this.ActivityProcessingStatusModels.Add(processingModel);
                    }
                }

                this.Breadcrumb = breadcrumb;

                this.ContainItems = this.ActivityProcessingStatusModels.Count > 0;

                this.View.Visibility = Visibility.Visible;
            }
            catch (TridentCustomException ex)
            {
                TridentMessageBox.ShowTridentErrorMessageBox(ex.Message);
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Event for property change.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raise property changed eventargs.
        /// </summary>
        /// <param name="changeProperty">Property anme.</param>
        private void OnNotifyPropertyChanged(string changeProperty)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(changeProperty));
            }
        }

        #endregion
    }
}
