//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.Provenance.Controls.Presenter
{
    using System;
    using Microsoft.Research.Provenance.Controls.View;
    using Microsoft.Practices.Unity;
    using Microsoft.Research.ProvenanceCommon.Model;
    using Microsoft.Research.ProvenanceInterfaces;
    using Microsoft.Research.ProvenanceCommon;
    using System.ComponentModel;
    using System.Windows.Input;
    using Microsoft.Research.ScientificWorkflow.UIDesigner.WpfComposite;
    using System.Collections.ObjectModel;
    using Microsoft.Research.Provenance.ProvenanceDataService;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;

    /// <summary>
    /// Presentation logic for displaying versions of the Workflow.
    /// </summary>
    public class VersionHistoryPresenter : IDisposable, INotifyPropertyChanged
    {
        private IUnityContainer container;

        /// <summary>
        /// Get SummaryModel
        /// </summary>
        public ObservableCollection<VersionHistoryModel> VersionModels { get; private set; }

        /// <summary>
        /// Get WorkFlowSummaryView
        /// </summary>
        public VersionHistoryView View { get; private set; }

        /// <summary>
        /// The command displays version history.
        /// </summary>
        public ICommand GetVersionHistory { get; private set; }

        /// <summary>
        /// This is to check whether the view is updated or not.
        /// </summary>
        public bool IsUpdated { get; private set; }

        /// <summary>
        /// This is used to set the expanding behaviour of the view.
        /// </summary>
        private bool isExpanded;

        /// <summary>
        /// Gets or sets the IsExpanded property.
        /// </summary>
        public bool IsExpanded
        {
            get
            {
                return this.isExpanded;
            }
            set
            {
                this.isExpanded = value;
                this.OnNotifyPropertyChanged("IsExpanded");
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="container">UnityContainer.</param>
        /// <param name="view">View related to version history.</param>
        public VersionHistoryPresenter(IUnityContainer container, VersionHistoryView view)
        {
            this.container = container;

            this.View = view;
            this.View.Presenter = this;
            this.VersionModels = new ObservableCollection<VersionHistoryModel>();

            this.GetVersionHistory = new DelegateCommand<object>(p => { this.OnGetVersionHistory(); });

            this.IsUpdated = false;
        }

        /// <summary>
        /// Get version details.
        /// </summary>
        private void OnGetVersionHistory()
        {
            try
            {
                Collection<VersionHistoryModel> versions = this.container.Resolve<DataRetriever>().GetWorkflowVersions();
                foreach (VersionHistoryModel version in versions)
                {
                    this.VersionModels.Add(version);
                }

                this.IsUpdated = true;
            }
            catch (TridentCustomException ex)
            {
                TridentMessageBox.ShowTridentErrorMessageBox(ex.Message);
            }
            catch (Exception exp)
            {
                TridentErrorHandler.HandleKnownExceptionsInUI(exp);
            }
        }

        #region IDisposable Members

        /// <summary>
        /// Clears the collection and View Object
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// nullify and Disposes the object 
        /// </summary>
        /// <param name="disposing"></param>
        private void Dispose(bool disposing)
        {
            if (disposing)
            {
                this.VersionModels = null;
                this.View = null;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Event for property change.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raise property changed eventargs.
        /// </summary>
        /// <param name="changeProperty">Property anme.</param>
        private void OnNotifyPropertyChanged(string changeProperty)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(changeProperty));
            }
        }

        #endregion
    }
}
