//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.Provenance.Controls.Presenter
{
    using System;
    using Microsoft.Practices.Unity;
    using Microsoft.Research.ProvenanceCommon.Model;
    using Microsoft.Research.Provenance.Controls.View;
    using Microsoft.Research.ProvenanceInterfaces;
    using Microsoft.Research.ProvenanceCommon;
    using System.Windows;
    using System.Collections.ObjectModel;
    using Microsoft.Research.DataLayer;
    using Microsoft.Research.ScientificWorkflow.UIDesigner.Monitor;
    using System.Collections.Generic;
    using System.Linq;
    using Microsoft.Research.Provenance.ProvenanceDataService;
    using System.ComponentModel;
    using System.Windows.Input;
    using Microsoft.Research.ScientificWorkflow.UIDesigner.WpfComposite;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;

    /// <summary>
    /// Presentation logic for displaying all the generated data 
    /// products of the Workflow.
    /// </summary>
    public class WorkflowDataProductPresenter : IDisposable, INotifyPropertyChanged
    {
        /// <summary>
        /// The unity container instance.
        /// </summary>
        private IUnityContainer container;

        /// <summary>
        /// 
        /// </summary>
        public ObservableCollection<DataProductItem> DataProductItemCollection { get; set; }

        /// <summary>
        /// Get WorkFlowSummaryView
        /// </summary>
        public WorkflowDataProductView View { get; private set; }

        /// <summary>
        /// Inidicates whether data is not present.
        /// </summary>
        private bool doesNotContainItems;

        /// <summary>
        /// Gets whether data is not present.
        /// </summary>
        public bool DoesNotContainItems
        {
            get
            {
                return this.doesNotContainItems;
            }
            private set
            {
                this.doesNotContainItems = value;
                this.OnNotifyPropertyChanged("DoesNotContainItems");
            }
        }

        /// <summary>
        /// Inidicates whether data is present.
        /// </summary>
        private bool containItems;

        /// <summary>
        /// Gets whether data is present.
        /// </summary>
        public bool ContainItems
        {
            get
            {
                return this.containItems;
            }
            private set
            {
                this.containItems = value;
                this.DoesNotContainItems = !this.ContainItems;
                this.OnNotifyPropertyChanged("ContainItems");
            }
        }

        /// <summary>
        /// The command displays Data products.
        /// </summary>
        public ICommand GetDataProducts { get; private set; }

        /// <summary>
        /// This is to check whether the view is updated or not.
        /// </summary>
        public bool IsUpdated { get; private set; }

        /// <summary>
        /// This is used to set the expanding behaviour of the view.
        /// </summary>
        private bool isExpanded;

        /// <summary>
        /// Gets or sets the IsExpanded property.
        /// </summary>
        public bool IsExpanded
        {
            get
            {
                return this.isExpanded;
            }
            set
            {
                this.isExpanded = value;
                this.OnNotifyPropertyChanged("IsExpanded");
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="container">UnityContainer.</param>
        /// <param name="view">View related to DataProduct Presenter.</param>
        public WorkflowDataProductPresenter(IUnityContainer container, WorkflowDataProductView view)
        {
            this.container = container;
            this.DataProductItemCollection = new ObservableCollection<DataProductItem>();
            this.GetDataProducts = new DelegateCommand<object>(p => { this.OnGetDataProducts(); });
            
            this.View = view;
            this.View.Presenter = this;
        }

        /// <summary>
        /// Event handler for getting the data products from Registry.
        /// </summary>
        private void OnGetDataProducts()
        {
            this.DataProductItemCollection.Clear();
            try
            {
                Collection<DataProductItem> tempList = this.container.Resolve<DataRetriever>().GetDataProducts();

                foreach (DataProductItem dataProduct in tempList)
                {
                    this.DataProductItemCollection.Add(dataProduct);
                }

                this.ContainItems = this.DataProductItemCollection.Count > 0;

                this.IsUpdated = true;
            }
            catch (TridentCustomException ex)
            {
                TridentMessageBox.ShowTridentErrorMessageBox(ex.Message);
            }
            catch (Exception exp)
            {
                TridentErrorHandler.HandleKnownExceptionsInUI(exp);
            }
        }

        #region IDisposable Members

        /// <summary>
        /// Clears the collection and View Object
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// nullify and Disposes the object 
        /// </summary>
        /// <param name="disposing"></param>
        private void Dispose(bool disposing)
        {
            if (disposing)
            {
                this.DataProductItemCollection = null;
                this.View = null;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Event for property change.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raise property changed eventargs.
        /// </summary>
        /// <param name="changeProperty">Property anme.</param>
        private void OnNotifyPropertyChanged(string changeProperty)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(changeProperty));
            }
        }

        #endregion
    }
}
