//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.Provenance.Controls.Presenter
{
    using System;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.Windows.Input;
    using Microsoft.Research.Provenance.Controls.View;
    using Microsoft.Research.Provenance.ProvenanceDataService;
    using Microsoft.Research.ProvenanceCommon.Model;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    using Microsoft.Research.ScientificWorkflow.UIDesigner.WpfComposite;
    using Microsoft.Practices.Unity;

    /// <summary>
    /// Presentation logic for displaying outputs of a Workflow.
    /// </summary>
    public class WorkflowOutputPresenter : IDisposable, INotifyPropertyChanged
    {
        /// <summary>
        /// This is used to set the expanding behaviour of the view.
        /// </summary>
        private bool isExpanded;

        /// <summary>
        /// The unity container instance.
        /// </summary>
        private IUnityContainer container;

        /// <summary>
        /// Inidicates whether data is not present.
        /// </summary>
        private bool doesNotContainItems;

        /// <summary>
        /// Inidicates whether data is present.
        /// </summary>
        private bool containItems;

        /// <summary>
        /// Gets or sets the output properties.
        /// </summary>
        /// <value>The output properties.</value>
        public ObservableCollection<ActivityProperty> OutputProperties { get; private set; }

        /// <summary>
        /// Gets or sets the view.
        /// </summary>
        /// <value>The view.</value>
        public WorkflowOutputView View { get; private set; }

        /// <summary>
        /// The command to get workflow outputs.
        /// </summary>
        public ICommand GetWorkflowOutputs { get; private set; }

        /// <summary>
        /// This is to check whether the view is updated or not.
        /// </summary>
        public bool IsUpdated { get; private set; }

        /// <summary>
        /// Gets or sets the IsExpanded property.
        /// </summary>
        public bool IsExpanded
        {
            get
            {
                return this.isExpanded;
            }

            set
            {
                this.isExpanded = value;
                this.OnNotifyPropertyChanged("IsExpanded");
            }
        }

        /// <summary>
        /// Gets whether data is not present.
        /// </summary>
        public bool DoesNotContainItems
        {
            get
            {
                return this.doesNotContainItems;
            }

            private set
            {
                this.doesNotContainItems = value;
                this.OnNotifyPropertyChanged("DoesNotContainItems");
            }
        }

        /// <summary>
        /// Gets whether data is present.
        /// </summary>
        public bool ContainItems
        {
            get
            {
                return this.containItems;
            }

            private set
            {
                this.containItems = value;
                this.DoesNotContainItems = !this.ContainItems;
                this.OnNotifyPropertyChanged("ContainItems");
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowOutputPresenter"/> class.
        /// </summary>
        /// <param name="container">The container.</param>
        /// <param name="view">The view.</param>
        public WorkflowOutputPresenter(IUnityContainer container, WorkflowOutputView view)
        {
            this.container = container;
            this.View = view;
            this.GetWorkflowOutputs = new DelegateCommand<object>(p => { this.OnGetWorkflowOutputs(); });
            this.OutputProperties = new ObservableCollection<ActivityProperty>();
            this.View.Presenter = this;
        }

        /// <summary>
        /// This function is used to get workflow outputs.
        /// </summary>
        private void OnGetWorkflowOutputs()
        {
            this.OutputProperties.Clear();
            try
            {
                Collection<ActivityProperty> tempList = this.container.Resolve<DataRetriever>().GetWorkflowOutputs();
                foreach (ActivityProperty property in tempList)
                {
                    this.OutputProperties.Add(property);
                }

                this.ContainItems = this.OutputProperties.Count > 0;
                this.IsUpdated = true;
            }
            catch (TridentCustomException ex)
            {
                TridentMessageBox.ShowTridentErrorMessageBox(ex.Message);
            }
            catch (Exception exp)
            {
                TridentErrorHandler.HandleKnownExceptionsInUI(exp);
            }
        }

        #region IDisposable Members

        /// <summary>
        /// Clears the collection and View Object
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources
        /// </summary>
        /// <param name="disposing"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        private void Dispose(bool disposing)
        {
            if (disposing)
            {
                this.container.Dispose();
                this.OutputProperties = null;
                this.View = null;
            }
        }

        #endregion

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Event for property change.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Raise property changed eventargs.
        /// </summary>
        /// <param name="changeProperty">Property anme.</param>
        private void OnNotifyPropertyChanged(string changeProperty)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(changeProperty));
            }
        }

        #endregion
    }
}
