//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using System;
using System.Collections.ObjectModel;
using System.Linq;
using Microsoft.Research.ScientificWorkflow.TridentUtilities;
using SR = Microsoft.Research.DataLayer;

namespace Microsoft.Research.ScientificWorkflow.TridentComposer
{
    /// <summary>
    /// Imports activities in a separate appdomain.
    /// </summary>
    public class ImportActivityProxy : MarshalByRefObject
    {
        #region private fields
        private Collection<FileReference> fileReferences;
        private SR.Connection registryConnection;
        private bool createNewVersionOnNameMatch;
        #endregion

        #region Constructors
        /// <summary>
        /// Dummy constructor. Needed to create this object by CreateInstanceAndUnWrap method.
        /// </summary>
        public ImportActivityProxy()
        {
        }

        /// <summary>
        /// Initializes a new instance of the Import activity proxy
        /// </summary>
        /// <param name="connection">The Connection.</param>
        /// <param name="references">The file references.</param>
        /// <param name="createNewVersionOnNameMatch">True if we need to create new version if the name matches. False otherwise.</param>
        public ImportActivityProxy(SR.Connection connection, Collection<FileReference> references, bool createNewVersionOnNameMatch)
        {
            if (references != null)
            {
                this.fileReferences = new Collection<FileReference>(references.ToList());
            }
            this.registryConnection = connection;
            this.createNewVersionOnNameMatch = createNewVersionOnNameMatch;
        }
        #endregion

        #region public methods
        /// <summary>
        /// Import activities.
        /// </summary>
        /// <returns>True if Import succeeded; False otherwise.</returns>
        public bool? Execute()
        {
            AppDomain childAppdomain = null;
            try
            {
                childAppdomain = this.CreateAppDomain();

                ImportActivityProxy importProxy = childAppdomain.CreateInstanceAndUnwrap
                    ("Microsoft.Research.ScientificWorkflow.TridentComposer",
                    "Microsoft.Research.ScientificWorkflow.TridentComposer.ImportActivityProxy") as ImportActivityProxy;
                importProxy.Init(this.registryConnection, this.fileReferences, this.createNewVersionOnNameMatch);
                return importProxy.ImportActivity();
            }
            finally
            {
                UnloadAppDomain(childAppdomain);
            }
        }
        #endregion

        #region private methods
        /// <summary>
        /// Initailize the new object created in another appdomain.
        /// </summary>
        /// <param name="registryConnection">The registry connection.</param>
        /// <param name="fileReferences">The file references.</param>
        /// <param name="createNewVersionOnNameMatch">True if we need to create new version if the name matches. False otherwise.</param>
        private void Init(SR.Connection registryConnection, Collection<FileReference> fileReferences, bool createNewVersionOnNameMatch)
        {
            this.registryConnection = registryConnection;
            this.fileReferences = fileReferences;
            this.createNewVersionOnNameMatch = createNewVersionOnNameMatch;

            SR.SR_Connection.Init();
            this.registryConnection.Open();
            this.registryConnection.AutoSave = false;

            // Authenticate the user.
            try
            {
                TridentAuthentication authenticate = new TridentAuthentication(this.registryConnection);
                authenticate.AuthenticateUser();
            }
            catch (TridentCustomException)
            {
                throw;
            }
        }

        /// <summary>
        /// Import the activity.
        /// </summary>
        /// <returns>True if Import succeeded; False otherwise.</returns>
        private bool? ImportActivity()
        {
            bool? importStatus = null;

            ImportActivity import = new ImportActivity(this.registryConnection, false);
            importStatus = import.ImportActivities(this.fileReferences, null, this.createNewVersionOnNameMatch);

            WorkflowComposer.ImportTypeInitializer(this.fileReferences, this.registryConnection);
            return importStatus;
        }

        /// <summary>
        /// Creates a new app domain.
        /// </summary>
        /// <returns>New instance of the app domain.</returns>
        private AppDomain CreateAppDomain()
        {
            AppDomain newDomain = AppDomain.CreateDomain("ImportActivityAppdomain");
            newDomain.UnhandledException += new UnhandledExceptionEventHandler(AppDomainUnhandledException);
            return newDomain;
        }

        /// <summary>
        /// Unloads the app domain.
        /// </summary>
        /// <param name="appDomain">Instance of the AppDomain.</param>
        private static void UnloadAppDomain(AppDomain appDomain)
        {
            if (appDomain != null)
            {
                AppDomain.Unload(appDomain);
            }
        }

        /// <summary>
        /// Method logs the unhandled exception in the app domain.
        /// </summary>
        /// <param name="sender">Object who raises the unhandled exception.</param>
        /// <param name="e">Event argument containing exception details.</param>
        private void AppDomainUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            // No-op.
        }
        #endregion
    }
}
