//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using Microsoft.Research.ScientificWorkflow;

namespace Microsoft.Research.ScientificWorkflow.TridentModel
{
    #region Using Directives
    using System;
    using System.Collections.Specialized;
    using System.ComponentModel;
    using System.Windows;
    using System.Workflow.ComponentModel;
    using Microsoft.Practices.EnterpriseLibrary.Logging;
    using SR = Microsoft.Research.DataLayer;
    using WfActivity = System.Workflow.ComponentModel.Activity;
    #endregion

    [Serializable]
    public class CompositeBlackboxModel : BaseCompositeModel
    {
        #region Private Member Data
        /// <summary>
        /// Whether BuiltIn Model.
        /// </summary>
        private bool isBuiltIn;

        /// <summary>
        /// This is used to check if the Activity Parameter Bindings are visible or not.
        /// </summary>
        private Visibility currentParameterBindingVisibility = Visibility.Visible;

        /// <summary>
        /// The execution status of the activity.
        /// </summary>
        private TridentActivityExecutionStatus currentTridentActivityExecutionStatus = TridentActivityExecutionStatus.None;
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activity">
        /// The blackbox activity to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        public CompositeBlackboxModel(
            WfActivity activity,
            BaseCompositeModel parentModel,
            ModelFactory modelFactory)
        {
            if (activity != null)
            {
                this.GetDetailsFromActivity(activity, parentModel, modelFactory);
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activitySequence">
        /// Activity sequence which has to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        public CompositeBlackboxModel(
            SR.ActivitySequence activitySequence,
            BaseCompositeModel parentModel,
            ModelFactory modelFactory)
        {
            if (activitySequence != null)
            {
                this.GetDetailsFromActivity(activitySequence, parentModel, modelFactory);
            }
        }
        
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="activitySequence">
        /// Activity sequence which has to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        public CompositeBlackboxModel(
            SR.Activity activity,
            BaseCompositeModel parentModel,
            string uniqueId)
        {
            if (activity != null)
            {
                this.GetDetailsFromActivity(activity, parentModel, uniqueId);
            }
        }

        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the visibility of the Bindings.
        /// </summary>
        public override System.Windows.Visibility CurrentParameterBindingVisibility
        {
            get
            {
                return this.currentParameterBindingVisibility;
            }
            set
            {
                this.currentParameterBindingVisibility = value;
            }
        }

        /// <summary>
        /// Gets and sets the CurrentActivityExecutionStatus.
        /// </summary>
        public TridentActivityExecutionStatus CurrentTridentActivityExecutionStatus
        {
            get
            {
                return this.currentTridentActivityExecutionStatus;
            }

            set
            {
                if (value != this.currentTridentActivityExecutionStatus)
                {
                    this.currentTridentActivityExecutionStatus = value;
                    this.RaisePropertyChanged("CurrentTridentActivityExecutionStatus");
                }
            }
        }

        /// <summary>
        /// Gets whether the Activity is of Built In Type.
        /// </summary>
        public bool IsBuiltIn
        {
            get
            {
                return this.isBuiltIn;
            }
        }

        /// <summary>
        /// A flag to check if the model is a simple activity model.
        /// </summary>
        public override bool IsSimpleActivityModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// A flag to check if the model is a composite activity model.
        /// </summary>
        public override bool IsCompositeActivityModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// A flag to check if the model is a subsection model.
        /// </summary>
        public override bool IsSubsectionModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// A flag to check if the model is a simpleblackboxmodel.
        /// </summary>
        public override bool IsSimpleBlackBoxModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// A flag to check if the model is a compositeblackboxmodel.
        /// </summary>
        public override bool IsCompositeBlackBoxModel
        {
            get
            {
                return true;
            }
        }

        /// <summary>
        /// A flag to check if the model is a foractivitymodel.
        /// </summary>
        public override bool IsForActivityModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// A flag to check if the model is a replicatoractivitymodel.
        /// </summary>
        public override bool IsReplicatorActivityModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        ///  A flag to check if the model is a ifelseactivitymodel.
        /// </summary>
        public override bool IsIfElseActivityModel
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// If false, the start and end elements are inserted, if true they are not inserted.
        /// </summary>
        public bool HideStartEnd { get; set; }
        #endregion        

        #region Public Methods
        /// <summary>
        /// Using the BaseVisitor, perform the operation.
        /// </summary>
        /// <param name="visitor">
        /// The implementation of visitor.
        /// </param>
        public override void Accept(TridentBaseVisitor visitor)
        {
            visitor.VisitCompositeBlackboxModel(this);
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Initialze the activity model, recursively, using the Activity object.
        /// </summary>
        /// <param name="activity">
        /// The blackbox activity to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent activity model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        /// <returns>
        /// Activitymodel representation for the ComponentModel.Activity passed.
        /// </returns>
        private void GetDetailsFromActivity(
            WfActivity activity,
            BaseCompositeModel parentActivityModel,
            ModelFactory modelFactory)
        {
            Type activityType = activity.GetType();
            this.UniqueId = activity.Name;
            this.ActivityClass = activityType.FullName;
            this.Id = Guid.Empty;

            // Get the label from the attribute.
            object[] attsNameLabel = null;
            try
            {
                attsNameLabel = activityType.GetCustomAttributes(typeof(NameAttribute), false);
            }
            catch (TypeLoadException ex)
            {
                Logger.Write(ex.Message);
            }

            this.Name = activityType.Name;
            if ((attsNameLabel == null) || ((attsNameLabel != null) && (attsNameLabel.Length == 0)))
            {
                this.Label = activityType.Name;
            }
            else
            {
                this.Label = ((NameAttribute)attsNameLabel[0]).Label;
            }

            // Get the description from the attribute.
            object[] attsDesc = null;
            try
            {
                attsDesc = activityType.GetCustomAttributes(typeof(DescriptionAttribute), false);
            }
            catch (TypeLoadException ex)
            {
                Logger.Write(ex.Message);
            }

            if (attsDesc != null && attsDesc.Length > 0)
            {
                this.Description = ((DescriptionAttribute)attsDesc[0]).Description;
            }

            this.IsDeleted = false;
            if (parentActivityModel != null)
            {
                this.ParentModel = parentActivityModel;
            }

            if (activityType.Namespace.Equals("System.Workflow.Activities") || activityType.Namespace.Equals("System.Workflow.ComponentModel"))
            {
                this.isBuiltIn = true;
            }
            else
            {
                this.isBuiltIn = false;
            }

            // Recurse through the children to get the child models.
            CompositeActivity compAct = activity as CompositeActivity;
            if (compAct != null)
            {
                this.Children.CollectionChanged += new NotifyCollectionChangedEventHandler(this.OnChildCollectionChanged);
                this.ChildrenCollectionChanged += new EventHandler<NotifyCollectionChangedEventArgs>(this.OnChildCollectionChanged);

                for (int index = 0; index < compAct.Activities.Count; index++)
                {
                    WfActivity childActivity = compAct.Activities[index];
                    BaseModel childModel = modelFactory.FetchModelByActivity(childActivity, this);
                    if (null != childModel)
                    {
                        // Add childActivity Model to the parent.
                        this.Children.Add(childModel);
                    }
                }
            }

        }

        /// <summary>
        /// Initialze the activity model, recursively, using the Activity Sequence object.
        /// </summary>
        /// <param name="activitySequence">
        /// Activity sequence which has to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        private void GetDetailsFromActivity(
            SR.ActivitySequence activitySequence,
            BaseCompositeModel parentActivityModel,
            ModelFactory modelFactory)
        {
            this.GetDetailsFromActivity(activitySequence.Activity, parentActivityModel, activitySequence.Name);

            // Recurse through the children to get the child models.
            for (int index = 0; index < activitySequence.Children.Count; index++)
            {
                SR.ActivitySequence childActivity = activitySequence.Children[index];

                if (childActivity != null)
                {
                    BaseModel childModel = modelFactory.FetchModelByActivitySequence(childActivity, this);
                    if (null != childModel)
                    {
                        // Add childActivity Model to the parent.
                        this.Children.Add(childModel);
                    }
                }
            }
        }

        /// <summary>
        /// Initialze the activity model, recursively, using the Activity Sequence object.
        /// </summary>
        /// <param name="activitySequence">
        /// Activity sequence which has to be converted to blackbox model.
        /// </param>
        /// <param name="parentModel">
        /// The parent model.
        /// </param>
        /// <param name="modelFactory">
        /// Model Factory.
        /// </param>
        private void GetDetailsFromActivity(
            SR.Activity erActivity,
            BaseCompositeModel parentActivityModel,
            string unqiueId)
        {
            this.UniqueId = unqiueId;

            this.ActivityClass = erActivity.ActivityClass;

            this.Id = ((SR.IObject)erActivity).ID;

            this.Name = erActivity.Name;

            this.Label = erActivity.Label;

            this.Description = erActivity.Description;

            this.IsDeleted = erActivity.IsDeleted;

            this.isBuiltIn = erActivity.IsBuiltIn;

            this.PopulateAssociatedAssemblies(erActivity);

            if (parentActivityModel != null)
            {
                this.ParentModel = parentActivityModel;
            }

            this.Children.CollectionChanged += new NotifyCollectionChangedEventHandler(this.OnChildCollectionChanged);
            this.ChildrenCollectionChanged += new EventHandler<NotifyCollectionChangedEventArgs>(this.OnChildCollectionChanged);
        }

        #endregion
    }
}
