//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.TridentUtilities
{
    #region NameSpace
    using System;
    using System.Globalization;
    using System.Runtime.Serialization;
    using System.Text;
    using System.Security.Permissions;
    #endregion

    [Serializable]
    public class TridentCustomException : Exception
    {
        #region Declaration
        /// <summary>
        /// custom message string
        /// </summary>
        private StringBuilder customMessage;

        /// <summary>
        /// Error caption string
        /// </summary>
        private string errorCaption;

        /// <summary>
        /// Error Number
        /// </summary>
        private double errorNumber;
        #endregion

        #region Constructor

        /// <summary>
        /// Constructor - Invokes default base Constructor
        /// </summary>
        public TridentCustomException()
            : base()
        {
        }

        /// <summary>
        /// Constructor - Invokes base Constructor
        /// </summary>
        /// <param name="message">string</param>
        public TridentCustomException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Constructor - Invokes base Constructor
        /// </summary>
        /// <param name="message">string</param>
        /// <param name="exception">Exception</param>
        public TridentCustomException(string message, Exception innerException)
            : base(message, innerException)
        {
        }

        /// <summary>
        /// Constructor - Invokes base Constructor
        /// </summary>
        /// <param name="serializationInfo">
        /// The System.Runtime.Serialization.SerializationInfo that holds the serialized
        ///     object data about the exception being thrown.
        /// </param>
        /// <param name="streamingContext">
        /// The System.Runtime.Serialization.StreamingContext that contains contextual
        ///     information about the source or destination.
        /// </param>
        protected TridentCustomException(
            SerializationInfo serializationInfo,
            StreamingContext streamingContext)
            : base(serializationInfo, streamingContext)
        {
            errorCaption = serializationInfo.GetString("ErrorCaption");
            customMessage = new StringBuilder(serializationInfo.GetString("CustomMessage"));
            errorNumber = Convert.ToDouble(serializationInfo.GetString("ErrorNumber"), CultureInfo.CurrentCulture);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="serializationInfo">
        /// The System.Runtime.Serialization.SerializationInfo that holds the serialized
        ///     object data about the exception being thrown.
        /// </param>
        /// <param name="streamingContext">
        /// The System.Runtime.Serialization.StreamingContext that contains contextual
        ///     information about the source or destination.
        /// </param>
        [SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        public override void GetObjectData(
            SerializationInfo info, 
            StreamingContext context)
        {
            if (info == null)
            {
                throw new ArgumentNullException("info");
            }

            info.AddValue("ErrorCaption", errorCaption);
            info.AddValue("CustomMessage", customMessage);
            info.AddValue("ErrorNumber", errorNumber);

            base.GetObjectData(info, context);
        }

        /// <summary>
        /// Constructor - Invokes default base Constructor
        /// </summary>
        /// <param name="paramErrorNumber">double</param>
        public TridentCustomException(double errorNumber)
            : base()
        {
            this.errorNumber = errorNumber;
        }

        /// <summary>
        /// Constructor - Invokes base Constructor
        /// </summary>
        /// <param name="paramErrorNumber">double</param>
        /// <param name="message">string</param>
        public TridentCustomException(double errorNumber, string message)
            : base(message)
        {
            this.errorNumber = errorNumber;
        }

        /// <summary>
        /// Constructor - Invokes base Constructor
        /// </summary>
        /// <param name="paramErrorNumber">double</param>
        /// <param name="message">string</param>
        /// <param name="exception">Exception</param>
        public TridentCustomException(double errorNumber, string message, Exception innerException)
            : base(message, innerException)
        {
            this.errorNumber = errorNumber;
        }
        #endregion

        #region Properties
        /// <summary>
        /// get and set  error caption
        /// </summary>
        public string ErrorCaption
        {
            get
            {
                return this.errorCaption;
            }
            set
            {
                this.errorCaption = string.Empty;
                if (!string.IsNullOrEmpty(value))
                {
                    this.errorCaption = value;
                }
            }
        }

        /// <summary>
        /// get and set Custom Error Message
        /// If errorNumber is non zero value it dispalays Error : Number  
        /// </summary>
        public StringBuilder CustomMessage
        {
            get
            {
                this.customMessage = new StringBuilder(); 
                if (0 == this.errorNumber)
                {
                    this.customMessage.Append(this.Message);
                }
                else
                {
                    this.customMessage.Append(this.errorCaption);
                    this.customMessage.Append(this.errorNumber);
                    this.customMessage.Append(" ");
                    this.customMessage.Append(this.Message);
                }

                return this.customMessage;
            }
        }

        /// <summary>
        /// Gets the Error Number.
        /// </summary>
        public string ErrorNumber
        {
            get
            {
                return this.errorNumber.ToString(CultureInfo.CurrentCulture);
            }
        }

        #endregion


    }
}
