//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner
{
    #region Namespace
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    using System.Windows.Media;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    #endregion

    // Need to add comment
    public class ActivityOutputParameterElement : ActivityParameterDesigner
    {
        /// <summary>
        /// The composer control holding this element.
        /// </summary>
        private ComposerControl composerCtrl;

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="param">ParameterDescriptionModel</param>
        /// <param name="parent">WorkflowActivityElement</param>
        public ActivityOutputParameterElement(ParameterDescriptionModel parameter, TridentUIActivityElement parent, BaseModel model, Dictionary<string, DataTypeModel> dataTypes)
            : base(parameter, parent, ParameterFlowType.OutputParameter, model)
        {
            this.Loaded += new RoutedEventHandler(this.OnLoaded);
            this.CreateOutPutParameter(parameter, dataTypes);
        }
        #endregion

        #region Private Method

        /// <summary>
        /// Creates output parameter 
        /// </summary>
        /// <param name="param">ParameterDescriptionModel</param>
        private void CreateOutPutParameter(ParameterDescriptionModel param, Dictionary<string, DataTypeModel> dataTypes)
        {
            this.ParamFillBrush = ActivityOutputParameterElement.CreateLightParamBrush(param, dataTypes); 
            this.AddParameterElementTextBlock(param.Label);
            this.AddParameterShape();
        }

        /// <summary>
        /// Get the datatype color
        /// </summary>
        /// <param name="param">ParameterDescriptionModel</param>
        /// <returns>SolidColorBrush</returns>
        private static SolidColorBrush CreateLightParamBrush(ParameterDescriptionModel param, Dictionary<string, DataTypeModel> dataTypes)
        {  
            // Check whether the Datatype is contained in the Datatypes[]
            Color dataypeColor;
            if (dataTypes.ContainsKey(param.DataType.ToString()))
            {
                DataTypeModel typeModel = dataTypes[param.DataType.ToString()];
                dataypeColor = typeModel.LightEditorColor;
            }
            else
            {
                // if it is not present then Default it to Purple.
                dataypeColor = Colors.Purple;
            }

            return new SolidColorBrush(dataypeColor);
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            this.GetComposerControl();
            this.ContextMenu = this.CreateContextMenu(this.ParamModel);
        }

        /// <summary>
        /// Get the composer control instance.
        /// </summary>
        private void GetComposerControl()
        {
            string composerCtrlName = (string)this.GetValue(ComposerControl.ComposerControlNameProperty);
            this.composerCtrl = Application.Current.MainWindow.FindName(composerCtrlName) as ComposerControl;
        }

        /// <summary>
        /// Creates the context menu.
        /// </summary>
        /// <param name="param">The param.</param>
        /// <returns></returns>
        private ContextMenu CreateContextMenu(ParameterDescriptionModel param)
        {
            if (this.composerCtrl != null && this.composerCtrl.CurrentRenderMode == RenderMode.ComposerMode)
            {
                MenuItem makeMandatory = new MenuItem();
                makeMandatory.Header = TridentResourceManager.GetString("MakeOutputMandatoryForWorkflowContextMenu");
                makeMandatory.Name = "makeMandatory";
                Binding mandatoryVisibilityBinding = new Binding("IsNotMandatoryForWorkflow");
                mandatoryVisibilityBinding.Mode = BindingMode.OneWay;
                mandatoryVisibilityBinding.Source = param;
                mandatoryVisibilityBinding.UpdateSourceTrigger = UpdateSourceTrigger.Default;
                mandatoryVisibilityBinding.Converter = new BooleanToVisibilityConverter();
                makeMandatory.SetBinding(MenuItem.VisibilityProperty, mandatoryVisibilityBinding);
                makeMandatory.Click += new RoutedEventHandler(this.OnMakeMandatoryClick);
                makeMandatory.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/mandatory.png");

                MenuItem makeOptional = new MenuItem();
                makeOptional.Header = TridentResourceManager.GetString("MakeOutputOptionalForWorkflowContextMenu");
                makeOptional.Name = "makeOptional";
                Binding optionalVisibilityBinding = new Binding("IsMandatoryForWorkflow");
                optionalVisibilityBinding.Mode = BindingMode.OneWay;
                optionalVisibilityBinding.Source = param;
                optionalVisibilityBinding.UpdateSourceTrigger = UpdateSourceTrigger.Default;
                optionalVisibilityBinding.Converter = new BooleanToVisibilityConverter();
                makeOptional.SetBinding(MenuItem.VisibilityProperty, optionalVisibilityBinding);
                makeOptional.Click += new RoutedEventHandler(this.OnMakeOptionalClick);
                makeOptional.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/optional.png");

                this.ContextMenu = new ContextMenu();
                this.ContextMenu.Items.Add(makeMandatory);
                this.ContextMenu.Items.Add(makeOptional);
            }

            return this.ContextMenu;
        }

        /// <summary>
        /// Called when context menu make mandatory click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void OnMakeMandatoryClick(object sender, RoutedEventArgs e)
        {
            this.UpdateMandatoryStatus(true);

            this.composerCtrl.CurrentComposerRenderHelper.ShowProperties(this.composerCtrl.CurrentWorkflowModel.Root, true);
        }

        /// <summary>
        /// Called when context menu make optional click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void OnMakeOptionalClick(object sender, RoutedEventArgs e)
        {
            this.UpdateMandatoryStatus(false);

            this.composerCtrl.CurrentComposerRenderHelper.ShowProperties(this.composerCtrl.CurrentWorkflowModel.Root, true);
        }

        /// <summary>
        /// Updates the mandatory status.
        /// </summary>
        /// <param name="isMandatory">if set to <c>true</c> the param model is marked as mandatory for workflow.</param>
        private void UpdateMandatoryStatus(bool isMandatory)
        {
            this.ParamModel.IsMandatoryForWorkflow = isMandatory;
            if (isMandatory)
            {
                this.ParamModel.OutputLabel = this.ParamModel.Name;
                this.ParamModel.IsFinal = true;
            }
            else
            {
                this.ParamModel.IsFinal = false;
            }

            this.AssignParameterBrush();
            if (this.composerCtrl != null)
            {
                this.composerCtrl.IsWFCommitted = false;
            }
        }

        #endregion
    }
}
