//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner
{
    #region Using Directives
    using System;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Shapes;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    #endregion

    /// <summary>
    /// This Class is used to represent the END 
    /// UI Element while rendering the workflows.
    /// </summary>
    public class EndElement : DockPanel, ITridentUIElement
    {
        #region Privtae Variables

        /// <summary>
        /// This is used to specify the Color of the border.
        /// </summary>
        private static Brush borderBrush = new SolidColorBrush(Colors.Transparent);

        /// <summary>
        /// This is used to specify the Color of the Connectivity Element.
        /// </summary>
        private static Brush flowBorderBrush = new SolidColorBrush(Colors.DarkCyan);

        /// <summary>
        /// This is used to specify the background Color of the UI Element as a whole.
        /// </summary>
        private static Brush flowFillBrush = new SolidColorBrush(Colors.Transparent);

        /// <summary>
        /// Thickness of the border.
        /// </summary>
        private static double flowShapeThickness = 1.5;

        /// <summary>
        /// Height of the Connectivity Element.
        /// </summary>
        private double flowShapeHeight = 10.0;

        /// <summary>
        /// Width of the Connectivity Element.
        /// </summary>
        private double flowShapeWidth = 10.0;

        /// <summary>
        /// Margin of the Connectivity Element.
        /// </summary>
        private static double flowShapeVertMargin = 1.0;

        /// <summary>
        /// Connectivity Element.
        /// </summary>
        private FrameworkElement leftFlowConnection;

        private bool isImageRequired = true;

        #endregion

        /// <summary>
        /// This is the Conctructor for this class.
        /// This is used to Initialize teh Ui Element.
        /// </summary>
        public EndElement(bool isImageRequired)
            : base()
        {
            this.isImageRequired = isImageRequired;
            if (!this.isImageRequired)
            {
                this.flowShapeHeight = 2;
                this.flowShapeWidth = 2;
            }

            this.InitComponent();
            this.SelectElement += new EventHandler(EndElement_SelectElement);
            this.DeleteElement += new EventHandler(EndElement_DeleteElement);
            this.ShowProperties += new EventHandler(EndElement_ShowProperties);
            this.UpdateClick += delegate(object sender, ModelEventArgs args) { };
            this.InsertIntoSubsection += new EventHandler(EndElement_InsertIntoSubsection);
            this.ProvenanceMenuClick += new EventHandler<ProvenanceModeEventArgs>(EndElement_ProvenanceMenuClick);
        }

        /// <summary>
        /// This is used to create an Ui Element for the End of the WorkFlow.
        /// </summary>
        private void InitComponent()
        {
            Border headerBorder = null;
            Height = 25;
            Width = 25;

            // Set the background of the UI Element.
            Background = new SolidColorBrush(Colors.Transparent);

            // This Creates the Body of the Element.
            // i.e. the Image and the Connectivity UI Element.
            UIElement header = this.InitBody();

            // Sets the Properties of the Header Border
            headerBorder = new Border();
            headerBorder.BorderBrush = borderBrush;
            headerBorder.BorderThickness = new Thickness(0, 0, 0, 1);
            headerBorder.Child = header;

            // Adds the Header to the Ui Element
            Children.Add(headerBorder);

            // Docks the Header to the TOP of teh Dockpanel.
            DockPanel.SetDock(headerBorder, Dock.Top);
        }

        /// <summary>
        /// Initializes the body of the Ui Element in the format
        /// the Connectivity Element and the Image.
        /// </summary>
        /// <returns>The Ui element created.</returns>
        private UIElement InitBody()
        {
            Grid headerPanel = null;
            ColumnDefinition flowInColumn = null;
            ColumnDefinition headerColumn = null;

            Canvas flowInCanvas = null;
            Image headerImage = null;
            Rectangle flowInRect = null;

            headerPanel = new Grid();

            // Sets the propeties of the Connectivity UI Element.
            flowInColumn = new ColumnDefinition();
            flowInColumn.Width = new GridLength(flowShapeWidth);

            // Sets the propeties of the Image UI Element.
            headerColumn = new ColumnDefinition();
            headerColumn.Width = new GridLength(1, GridUnitType.Star);

            #region Set the Value for the Connectivity UI Element

            // Adds the Connectivity UI Element to the Body
            headerPanel.ColumnDefinitions.Add(flowInColumn);

            flowInCanvas = new Canvas();
            flowInCanvas.Height = flowShapeWidth;
            flowInCanvas.VerticalAlignment = VerticalAlignment.Center;

            flowInRect = new Rectangle();
            flowInRect.Width = flowShapeWidth;
            flowInRect.Height = flowShapeHeight;
            flowInRect.Stroke = flowBorderBrush;
            flowInRect.Fill = flowFillBrush;
            flowInRect.StrokeThickness = flowShapeThickness;
            flowInRect.Stroke = new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FF94B3C4"));
            this.leftFlowConnection = flowInRect;
            #endregion

            #region Set the Value for the Image UI Element
            // Adds the Image Element to the Body
            headerPanel.ColumnDefinitions.Add(headerColumn);

            // Sets teh source for the End Image.
            headerImage = new Image();
            ImageSourceConverter converter = new ImageSourceConverter();
            headerImage.Source = (ImageSource)converter.ConvertFromString(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/WFEnd.PNG");
            headerImage.Height = 40;

            headerPanel.HorizontalAlignment = HorizontalAlignment.Stretch;
            headerPanel.VerticalAlignment = VerticalAlignment.Stretch;
            headerPanel.Margin = new Thickness(0, 0, 0, 0);
            #endregion

            // Sets the Padding.
            Canvas.SetTop(flowInRect, -(flowShapeVertMargin / 2.0));
            Canvas.SetLeft(headerImage, -(flowShapeWidth / 2.0));

            Grid.SetColumn(flowInCanvas, 0);
            Grid.SetColumn(headerImage, 1);

            flowInCanvas.Children.Add(flowInRect);

            // Adds the Connectivty element to the Main Grid
            headerPanel.Children.Add(flowInCanvas);

            if (isImageRequired)
            {
                // Adds the End Image element to the Main Grid
                headerPanel.Children.Add(headerImage);
            }

            return headerPanel;
        }

        #region ITridentUIElement Members

        public void ApplySelectedStyle()
        {
            throw new NotImplementedException();
        }

        public void ApplyUnselectedStyle()
        {
            throw new NotImplementedException();
        }

        public void CreateContextMenu()
        {
            throw new NotImplementedException();
        }

        public event EventHandler DeleteElement;

        public Point GetBindingEndpoint(UIElement relativeTo)
        {
            throw new NotImplementedException();
        }

        public Point GetBindingStartPoint(UIElement relativeTo)
        {
            throw new NotImplementedException();
        }

        public int FetchUIElementPosition()
        {
            throw new NotImplementedException();
        }

        public BaseModel Model
        {
            get { return null; }
        }

        public event EventHandler SelectElement;

        public event EventHandler ShowProperties;

        public event EventHandler InsertIntoSubsection;

        public event EventHandler<ProvenanceModeEventArgs> ProvenanceMenuClick;

        public event EventHandler<ModelEventArgs> UpdateClick;

        public FrameworkElement LeftControlFlowConnectionElement
        {
            get { return this.leftFlowConnection; }
        }

        public FrameworkElement RightControlFlowConnectionElement
        {
            get { return this.leftFlowConnection; }
        }

        #endregion

        private void EndElement_DeleteElement(object sender, EventArgs e)
        {
            // Note: Null implementation to avoid warning
        }

        private void EndElement_SelectElement(object sender, EventArgs e)
        {
            // Note: Null implementation to avoid warning
        }

        private void EndElement_ShowProperties(object sender, EventArgs e)
        {

        }

        private void EndElement_InsertIntoSubsection(object sender, EventArgs e)
        {

        }

        private void EndElement_ProvenanceMenuClick(object sender, ProvenanceModeEventArgs e)
        {
            // Note: Null implementation to avoid warning
        }
    }
}

