//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
    using System.Windows.Controls;
    using System.Windows;
    using System.Windows.Media;
    using System.Windows.Shapes;
    using System.Windows.Input;
    using System.Resources;
    using System.Collections.ObjectModel;
    using Microsoft.Research.eResearch.Common.Eventing;

    /// <summary>
    /// The Ui representation for a blackbox activity.
    /// </summary>
    public class CompositeBlackboxActivityElement : TridentUIActivityElement, ICompositeUIElement
    {
        /// <summary>
        /// The offset between the ChildElements collection and the MainContentHolder's children collection.
        /// This is due to the insertion of start element.
        /// </summary>
        private readonly int MetaUiElementsOffset = 1;

        /// <summary>
        /// Holds the main content.
        /// </summary>
        private StackPanel stkMainContentHolder;

        /// <summary>
        /// The stack header.
        /// </summary>
        private WorkflowStackHeader header;

        /// <summary>
        /// If true, the curremt model is a root element.
        /// </summary>
        private bool isRoot;

        /// <summary>
        /// The children of this composite element.
        /// </summary>
        private ObservableCollection<ITridentUIElement> childElements = new ObservableCollection<ITridentUIElement>();

        /// <summary>
        /// 
        /// </summary>
        private Brush borderColor =  new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FFC6D3D8"));
       
        /// <summary>
        /// Occurs when update is clicked.
        /// </summary>
        public override event EventHandler<ModelEventArgs> UpdateClick;

        /// <summary>
        /// UI representation for the composite activity.
        /// </summary>
        /// <param name="activity">
        /// The composite activity that the stack represents.
        /// </param>
        /// <param name="model">
        /// The activity model for the composite activity.
        /// </param>
        public CompositeBlackboxActivityElement(CompositeBlackboxModel model)
            : base(model)
        {
            this.isRoot = (model.ParentModel == null);
            this.MetaUiElementsOffset = (model.HideStartEnd ? 0 : 1);
            InitializeComponent();
        }

        /// <summary>
        /// The header text for the activity.
        /// </summary>
        public string HeaderText
        {
            get
            {
                if (this.header != null)
                {
                    return this.header.HeaderText;
                }
                return null;
            }
            set
            {
                if (this.header != null)
                {
                    this.header.HeaderText = value;
                }
            }
        }

        /// <summary>
        /// The stackpanel which holds the child activities.
        /// </summary>
        public StackPanel MainContentHolder
        {
            get { return stkMainContentHolder; }
            set { stkMainContentHolder = value; }
        }

        /// <summary>
        /// Intialize the required component
        /// </summary>
        public new void InitializeComponent()
        {
            base.InitializeComponent();

            // Add the border which will contain the composite activity.
            this.MainBorder = new Border();
            this.MainBorder.VerticalAlignment = VerticalAlignment.Top;
            this.MainBorder.BorderBrush = this.borderColor;
            this.MainBorder.BorderThickness = new Thickness(1, 1, 2, 2);
            this.MainBorder.CornerRadius = new CornerRadius(7, 7, 7, 7);

            // Make the UIElement to be focusable.
            // this is used when we select a Binding and hit delete button.
            this.MainBorder.Focusable = true;

            // The background of the container is set to make it selectable.
            // if you have not set any background to any WPF control it will not raise any mouse down events.
            // This is by design of the controls in WPF.
            this.MainBorder.Background = Brushes.Transparent;

            StackPanel compositeActivityHolder = new StackPanel();

            // The header of the Ui element.
            this.header = new WorkflowStackHeader(this.isRoot, this.Model);
            ImageSourceConverter converter = new ImageSourceConverter();

            this.header.HeaderImage = (ImageSource)converter.ConvertFromString(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/SeqActivity.png");
            this.header.HeaderBackground = Brushes.AliceBlue;
            this.header.HeaderText = this.Model.Label;
            this.header.UpdateClick += delegate(object sender, ModelEventArgs args) { this.UpdateClick.Fire(sender, args); };

            // Set the left and right connectors for the activity.
            this.LeftControlFlowConnectionElement = this.header.LeftControlFlowConnectionElement;
            this.RightControlFlowConnectionElement = this.header.RightControlFlowConnectionElement;

            compositeActivityHolder.Children.Add(this.header);

            // The body of the Element.
            Grid activityBody = new Grid();
            activityBody.MinHeight = 50;

            this.stkMainContentHolder = new StackPanel();
            activityBody.Children.Add(stkMainContentHolder);
         
            this.stkMainContentHolder.Orientation = Orientation.Horizontal;
            
            activityBody.Background = Brushes.Transparent;
            activityBody.Margin = new Thickness(5);

            compositeActivityHolder.Children.Add(activityBody);
            this.MainBorder.Child = compositeActivityHolder;
            this.contentHolder.Children.Add(this.MainBorder);
            
            this.Margin = new Thickness(30, 5, 30, 5);
            
        }

        /// <summary>
        /// Apply seleted style to the workflow activity element.
        /// </summary>
        public override void ApplySelectedStyle()
        {
            // Apply the background of the border.
            this.MainBorder.BorderBrush = Brushes.Orange;
        }

        /// <summary>
        /// Apply Unseleted stlye to the workflow activity element.
        /// </summary>
        public override void ApplyUnselectedStyle()
        {
            // Revert back all the changes made while selecting.
            this.MainBorder.BorderBrush = new SolidColorBrush((Color)ColorConverter.ConvertFromString("#FFC6D3D8"));
        }
         
        #region ICompositeUiElement Members

        public ObservableCollection<ITridentUIElement> ChildElements
        {
            get
            {
                return this.childElements;
            }
        }

        public void AddChildElement(ITridentUIElement childElement)
        {
            #region Parameters check.

            if (null == childElement)
            {
                throw new ArgumentNullException("childElement");
            }

            #endregion Parameters check.

            this.childElements.Add(childElement);
            UIElement uiElem = childElement as UIElement;
            if (uiElem != null)
            {
                this.stkMainContentHolder.Children.Add(uiElem);
            }
        }

        public void InsertChildElement(ITridentUIElement childElement, int indexToInsertAt)
        {
            #region Parameters check.

            if (null == childElement)
            {
                throw new ArgumentNullException("childElement");
            }

            if (0 > indexToInsertAt)
            {
                throw new ArgumentOutOfRangeException("indexToInsertAt");
            }

            #endregion Parameters check.

            this.childElements.Insert(indexToInsertAt, childElement);
            UIElement uiElem = childElement as UIElement;
            if (uiElem != null)
            {
                this.stkMainContentHolder.Children.Insert(indexToInsertAt + this.MetaUiElementsOffset, uiElem);
            }
        }

        public bool RemoveChildElement(ITridentUIElement childElement)
        {
            #region Parameters check.

            if (null == childElement)
            {
                throw new ArgumentNullException("childElement");
            }

            #endregion Parameters check.

            bool opStatus = false;
            if (this.childElements.Contains(childElement))
            {
                opStatus = this.childElements.Remove(childElement);
            }

            if (opStatus)
            {
                UIElement uiElem = childElement as UIElement;
                if (uiElem != null && this.stkMainContentHolder.Children.Contains(uiElem))
                {
                    this.stkMainContentHolder.Children.Remove(uiElem);
                }
            }

            return opStatus;
        }

        public void RemoveChildElementAt(int indexOfChild)
        {
            #region Parameters check.

            if (0 > indexOfChild || (this.childElements.Count - 1) < indexOfChild)
            {
                throw new ArgumentOutOfRangeException("indexOfChild");
            }

            if (0 > indexOfChild || (this.stkMainContentHolder.Children.Count - 1) < indexOfChild)
            {
                throw new ArgumentOutOfRangeException("indexOfChild");
            }

            #endregion Parameters check.

            this.childElements.RemoveAt(indexOfChild);
            this.stkMainContentHolder.Children.RemoveAt(indexOfChild + this.MetaUiElementsOffset);
        }

        Panel ICompositeUIElement.MainContentHolder
        {
            get { return this.stkMainContentHolder; }
        }

        #endregion

        public override Point GetBindingStartPoint(UIElement relativeTo)
        {
            return new Point();
        }

        public override Point GetBindingEndpoint(UIElement relativeTo)
        {
            return new Point();
        }
    }
}
