//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner
{
    #region NameSpace
    using System;
    using System.Collections.Generic;
    using System.Text;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
    using System.Windows.Controls;
    using System.Windows;
    using System.Windows.Media;
    using System.Windows.Shapes;
    using System.Windows.Input;
    using System.Resources;
    using System.Collections.ObjectModel;
    using System.Workflow.ComponentModel;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    #endregion

    /// <summary>
    /// The Ui representation for a blackbox activity.
    /// </summary>
    public class SimpleBlackboxActivityElement : TridentUIActivityElement
    {
        #region Declaration

        private SimpleBlackboxActivityBody body;
        private ActivityHeader header;
        private SimpleBlackboxModel activityModel;
        private Collection<MenuItem> activitySpecificMenuItems = new Collection<MenuItem>();
        #endregion

        public SimpleBlackboxActivityElement(SimpleBlackboxModel model, Dictionary<String, DataTypeModel> activityDataTypes)
            : base(model, activityDataTypes)
        {
            this.DataContext = model;
            this.activityModel = model;
            if (!string.IsNullOrEmpty(model.UniqueId))
            {
                this.Name = model.UniqueId + "Element";
            }

            InitComponent();
        }

        #region Public methods


        /// <summary>
        /// Extend the context menu to have specific item for this.
        /// </summary>
        public override void CreateContextMenu()
        {
            base.CreateContextMenu();
            if (this.Model.IsSimpleActivityModel && this.ContextMenu != null)
            {
                this.ContextMenu.Items.Add(this.ActivityShowHideContextMenu());
                this.ContextMenu.Opened += new RoutedEventHandler(this.OnContextMenuOpened);
            }
        }

        /// <summary>
        /// Override to support different handling for the monitor mode.
        /// </summary>
        protected override void AddInteractionHandlers()
        {
            if (this.ComposerCtrl != null)
            {
                if (this.ComposerCtrl.CurrentRenderMode == RenderMode.ComposerMode
                    || this.ComposerCtrl.CurrentRenderMode == RenderMode.ProvenanceMode)
                {
                    base.AddInteractionHandlers();
                }
                else if (this.ComposerCtrl.CurrentRenderMode == RenderMode.MonitorMode ||
                    this.ComposerCtrl.CurrentRenderMode == RenderMode.VisualizationMode)
                {
                    this.ContextMenu = null;

                    // Collapse/Show Menu is not required for Composite activity
                    if (this.header != null && this.body != null && this.activityModel != null && this.activityModel.IsSimpleActivityModel)
                    {
                        this.body.ContextMenu = new ContextMenu();
                        this.header.ContextMenu = new ContextMenu();
                        this.header.ContextMenu.Items.Add(this.ActivityShowHideContextMenu());
                        this.body.ContextMenu.Items.Add(this.ActivityShowHideContextMenu());
                    }
                }
            }
        }

        private void OnContextMenuOpened(object sender, RoutedEventArgs e)
        {
            if (this.ComposerCtrl != null)
            {
                bool singleSelection = this.ComposerCtrl.CurrentSelectedElements.Count > 1 ? false : true;
                foreach (MenuItem item in this.activitySpecificMenuItems)
                {
                    item.Visibility = singleSelection ? Visibility.Visible : Visibility.Collapsed;
                }
            }
        }

        /// <summary>
        /// Collapse hide Activity menu
        /// </summary>
        /// <returns>MenuItem</returns>
        private MenuItem ActivityShowHideContextMenu()
        {
            MenuItem activityCollapsedMenu = new MenuItem();
            if (this.Model.IsCollapsed)
            {
                activityCollapsedMenu.Header = TridentResourceManager.GetString("ShowActivity");
                activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/ShowActivities.png");
            }
            else
            {
                activityCollapsedMenu.Header = TridentResourceManager.GetString("CollapseActivity");
                activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/HideActivities.png");
            }

            activityCollapsedMenu.Click += new RoutedEventHandler(this.OnSimpleActivityMenuClick);
            activityCollapsedMenu.Loaded += new RoutedEventHandler(this.OnSimpleActivityMenuLoaded);
            this.activitySpecificMenuItems.Add(activityCollapsedMenu);
            return activityCollapsedMenu;
        }

        /// <summary>
        /// Event handler for the Context menu item loaded event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSimpleActivityMenuLoaded(object sender, RoutedEventArgs e)
        {
            MenuItem activityCollapsedMenu = sender as MenuItem;
            if (null != activityCollapsedMenu)
            {
                if (this.Model.IsCollapsed)
                {
                    activityCollapsedMenu.Header = TridentResourceManager.GetString("ShowActivity");
                    activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/ShowActivities.png");

                }
                else
                {
                    activityCollapsedMenu.Header = TridentResourceManager.GetString("CollapseActivity");
                    activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/HideActivities.png");
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSimpleActivityMenuClick(object sender, RoutedEventArgs e)
        {
            MenuItem activityCollapsedMenu = sender as MenuItem;
            if (null != activityCollapsedMenu)
            {
                if (activityCollapsedMenu.Header.ToString().Equals(TridentResourceManager.GetString("ShowActivity")))
                {
                    this.ExpandBody();
                    activityCollapsedMenu.Header = TridentResourceManager.GetString("CollapseActivity");
                    activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/HideActivities.png");
                }
                else
                {
                    this.CollapseBody();
                    activityCollapsedMenu.Header = TridentResourceManager.GetString("ShowActivity");
                    activityCollapsedMenu.Icon = TridentUIActivityElement.GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/ShowActivities.png");
                }

                if (this.ComposerCtrl != null)
                {
                    this.ComposerCtrl.RefreshUI();
                }
            }
        }

        #endregion

        /// <summary>
        /// Initialize the Activity component.
        /// </summary>
        private void InitComponent()
        {
            this.header = this.InitHeader(this.activityModel);
            DockPanel.SetDock(header, Dock.Top);

            this.body = SimpleBlackboxActivityElement.InitBody(this.activityModel);

            Grid childGrid = new Grid();
            childGrid.Children.Add(body);
            childGrid.Children.Add(header);

            this.MainBorder = new Border();
            this.MainBorder.Child = childGrid;
            this.MainBorder.BorderThickness = new Thickness(1.5);
            this.MainBorder.Padding = new Thickness(10, 10, 10, 0);

            this.contentHolder.Children.Add(MainBorder);

            // Collapse the body if the flag is set to true.
            if (this.activityModel.IsCollapsed)
            {
                this.CollapseBody();
            }
        }

        /// <summary>
        /// Initialize the header of the activity element.
        /// </summary>
        /// <param name="model">The activity model.</param>
        /// <returns>The control created.</returns>
        private ActivityHeader InitHeader(SimpleBlackboxModel model)
        {
            ActivityHeader activityHeader = new ActivityHeader(model);
            this.LeftControlFlowConnectionElement = activityHeader.rectangleLeft;
            this.RightControlFlowConnectionElement = activityHeader.rectangleRight;
            activityHeader.HeaderText = model.Label;
            activityHeader.OnOptionsClick += new EventHandler(this.OnActivityOptionsClick);

            if (model.IsBuiltIn)
            {
                GradientStop gs1 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFEEFADE"), 0);
                GradientStop gs2 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFD3E8AB"), 1);
                GradientStopCollection gsColl = new GradientStopCollection(2);
                gsColl.Add(gs1);
                gsColl.Add(gs2);

                LinearGradientBrush lnrGradBrush = new LinearGradientBrush(gsColl, new Point(0.5, 0), new Point(0.5, 1));
                activityHeader.HeaderBackground = lnrGradBrush;
            }

            return activityHeader;
        }

        /// <summary>
        /// Initialize the body of the activity.
        /// </summary>
        /// <param name="activity">The activity model.</param>
        /// <returns>The control created.</returns>
        private static SimpleBlackboxActivityBody InitBody(SimpleBlackboxModel model)
        {
            SimpleBlackboxActivityBody activityBody = new SimpleBlackboxActivityBody(model);
            if (model.IsBuiltIn)
            {
                GradientStop gs1 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFFBFFF3"), 0);
                GradientStop gs2 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFE8F6D2"), 1);
                GradientStopCollection gsColl = new GradientStopCollection(2);
                gsColl.Add(gs1);
                gsColl.Add(gs2);
                LinearGradientBrush lnrGradBrush = new LinearGradientBrush(gsColl, new Point(0.5, 0), new Point(0.5, 1));
                activityBody.BodyBackground = lnrGradBrush;
            }
            else
            {
                GradientStop gs1 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFFFFFFF"), 0);
                GradientStop gs2 = new GradientStop((Color)ColorConverter.ConvertFromString("#FFECF0F1"), 1);
                GradientStopCollection gsColl = new GradientStopCollection(2);
                gsColl.Add(gs1);
                gsColl.Add(gs2);
                LinearGradientBrush lnrGradBrush = new LinearGradientBrush(gsColl, new Point(0.5, 0), new Point(0.5, 1));
                activityBody.BodyBackground = lnrGradBrush;
            }


            return activityBody;
        }

        /// <summary>
        /// Fired on click of the options button.
        /// </summary>
        /// <param name="sender">
        /// Options button.
        /// </param>
        /// <param name="e">
        /// EventArgs.
        /// </param>
        private void OnActivityOptionsClick(object sender, EventArgs e)
        {
            if (this.ComposerCtrl != null)
            {
                this.ComposerCtrl.CurrentSelectedElements.RemoveAllItems();
                this.ComposerCtrl.CurrentSelectedElements.SelectChildrenUpTo(this);
            }

            if (this.ContextMenu != null)
            {
                this.ContextMenu.IsOpen = true;
            }
            if (this.ComposerCtrl != null && 
                (this.ComposerCtrl.CurrentRenderMode == RenderMode.MonitorMode || 
                this.ComposerCtrl.CurrentRenderMode == RenderMode.VisualizationMode))
            {
                if (this.body.ContextMenu != null)
                {
                    this.body.ContextMenu.IsOpen = true;
                }
            }
        }

        /// <summary>
        /// Toggles the parameter bindings.
        /// </summary>
        /// <param name="sender">
        /// Parameter element.
        /// </param>
        /// <param name="e">
        /// EventArgs.
        /// </param>
        private void OnToggleParameterBindings(object sender, EventArgs e)
        {
            if (this.activityModel.CurrentParameterBindingVisibility == Visibility.Collapsed)
            {
                base.ToggleParameterBindings(Visibility.Visible);
            }
            else
            {
                base.ToggleParameterBindings(Visibility.Collapsed);
            }
        }

        /// <summary>
        /// Updates the execution icon for the element
        /// </summary>
        /// <param name="imageSource"></param>
        private void UpdateExecutionIcon(string imageSource)
        {
            DockPanel body = ((DockPanel)((DockPanel)this.contentHolder.Children[1]).Children[0]);
            body.Children.Clear();
            Canvas iconPanel = new Canvas();
            Image icon = new Image();

            ImageSourceConverter converter = new ImageSourceConverter();
            icon.Source = (ImageSource)converter.ConvertFromString(imageSource);
            icon.Height = 30;

            iconPanel.Height = 0;

            body.Children.Add(iconPanel);
            iconPanel.Children.Add(icon);

            Canvas.SetBottom(icon, -15);
            Canvas.SetLeft(icon, -15);

            DockPanel.SetDock(iconPanel, Dock.Bottom);
        }

        /// <summary>
        /// Collapse the body.
        /// </summary>
        private void CollapseBody()
        {
            if (this.body != null)
            {
                this.body.Visibility = Visibility.Collapsed;
                this.activityModel.IsCollapsed = true;
            }
        }

        /// <summary>
        /// Maximize the body.
        /// </summary>
        private void ExpandBody()
        {
            if (this.body != null)
            {
                this.body.Visibility = Visibility.Visible;
                this.activityModel.IsCollapsed = false;
            }
        }

        public override void ApplySelectedStyle()
        {
            // Apply the background of the border.
            this.MainBorder.BorderBrush = Brushes.Orange;

            this.MainBorder.CornerRadius = new CornerRadius(5);
            this.MainBorder.VerticalAlignment = VerticalAlignment.Top;
        }

        public override void ApplyUnselectedStyle()
        {
            // Revert back all the changes made while selecting.
            this.MainBorder.BorderBrush = new SolidColorBrush(Colors.Transparent);

            this.MainBorder.CornerRadius = new CornerRadius(0);
            this.MainBorder.VerticalAlignment = VerticalAlignment.Top;
        }

        /// <summary>
        /// This function is used to update the Icon of the Parameter Binding image in 
        /// workflow activity element based on the visibilty of the parameter bindings.
        /// </summary>
        /// <param name="changeVisibilty">Visibilty of the cativity model.</param>
        public void UpdateParameterBindingDetail(Visibility changeVisibility)
        {
            if (null != this.header)
            {
                MenuItem hideParamBindingsMenu = this.GetParameterMenuItem();
                if (hideParamBindingsMenu != null)
                {
                    if (Visibility.Visible == changeVisibility)
                    {
                        hideParamBindingsMenu.Header = TridentResourceManager.GetString("HideParameterBindingText");
                        hideParamBindingsMenu.Icon = GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/HideParamBindings.png");
                    }
                    else
                    {
                        hideParamBindingsMenu.Header = TridentResourceManager.GetString("ShowParameterBindingText");
                        hideParamBindingsMenu.Icon = GetIcon(@"pack://Application:,,,/Microsoft.Research.ScientificWorkflow.TridentUtilities;component/Images/ScientificWorkflow/ShowParamBindings.png");
                    }
                }
            }
        }

        /// <summary>
        ///Retrives the  ParameterBinding Menu. from  the context menu
        /// </summary>
        /// <returns>MenuItem</returns>
        private MenuItem GetParameterMenuItem()
        {
            if (this.ContextMenu != null)
            {
                foreach (MenuItem menu in this.ContextMenu.Items)
                {
                    if (TridentResourceManager.GetString("HideParameterBindingText").Equals(menu.Header.ToString())
                        || TridentResourceManager.GetString("ShowParameterBindingText").Equals(menu.Header.ToString()))
                    {
                        return menu;
                    }

                }
            }
            return null;
        }

        public override Point GetBindingStartPoint(UIElement relativeTo)
        {
            if (this.activityModel.IsCollapsed)
            {
                // Get the start point in the activity header.
                Point startPoint = new Point(this.header.ActualWidth + 10, this.header.ActualHeight / 2.0 + 10.0);
                return this.TranslatePoint(startPoint, relativeTo);
            }
            else
            {
                // Get the midpoint of the outline.
                SimpleBlackboxActivityBody body = this.body as SimpleBlackboxActivityBody;
                if (body != null)
                {
                    return body.GetOutputBorderConnectionPosition(relativeTo);
                }
                else
                {
                    return new Point();
                }
            }
        }

        public override Point GetBindingEndpoint(UIElement relativeTo)
        {
            if (this.activityModel.IsCollapsed)
            {
                // Get the start point in the activity header.
                Point startPoint = new Point(10, this.header.ActualHeight / 2.0 + 10.0);
                return this.TranslatePoint(startPoint, relativeTo);
            }
            else
            {
                // Get the midpoint of the outline.
                SimpleBlackboxActivityBody body = this.body as SimpleBlackboxActivityBody;
                if (body != null)
                {
                    return body.GetInputBorderConnectionPosition(relativeTo);
                }
                else
                {
                    return new Point();
                }
            }
        }
    }
}


