//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//********************************************************* 

using System;
using System.ComponentModel;
using System.Windows.Forms;
using Microsoft.TridentWordAddIn.Common;

namespace Microsoft.TridentWordAddIn.Presentation
{
    internal partial class AccessKey : UserControl
    {
        #region get set methods

        /// <summary>
        /// Initializes a new instance of the <see cref="AccessKey"/> class.
        /// </summary>
        public AccessKey()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// Occurs when [on validation].
        /// </summary>
        public event EventHandler<ConnectionValidateEventArgs> OnValidation;

        /// <summary>
        /// Gets or sets the access key ID.
        /// </summary>
        /// <value>The access key ID.</value>
        public string AccessKeyID
        {
            get
            {
                return this.txtAccessKeyId.Text;
            }
            set
            {
                this.txtAccessKeyId.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the secret access key.
        /// </summary>
        /// <value>The secret access key.</value>
        public string SecretAccessKey
        {
            get
            {
                return this.txtSecretAccessKey.Text;
            }
            set
            {
                this.txtSecretAccessKey.Text = value;
            }
        }

        #endregion 

        #region Events

        /// <summary>
        /// Handles the Validating event of the txtAccessKeyId control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void txtAccessKeyId_Validating(object sender, CancelEventArgs e)
        {
            if (this.OnValidation != null)
            {
                ValidationResult result = this.IsValidAccessKey();
                ConnectionValidateEventArgs connectionValidateEventArgs = new ConnectionValidateEventArgs(result.GetMessage(), result.IsValid, "TextBox");
                this.OnValidation(sender, connectionValidateEventArgs);
            }
        }

        /// <summary>
        /// Handles the Validating event of the txtSecretAccessKey control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void txtSecretAccessKey_Validating(object sender, CancelEventArgs e)
        {
            if (this.OnValidation != null)
            {
                ValidationResult result = this.IsValidSecretAccessKey();
                ConnectionValidateEventArgs connectionValidateEventArgs = new ConnectionValidateEventArgs(result.GetMessage(), result.IsValid, "TextBox");
                this.OnValidation(sender, connectionValidateEventArgs);
            }
        }

        #endregion

        /// <summary>
        /// Determines whether this instance is valid.
        /// </summary>
        /// <returns></returns>
        public ValidationResult IsValid()
        {
            ValidationResult result = this.IsValidAccessKey();
            if (!result.IsValid)
            {
                this.txtAccessKeyId_Validating(txtAccessKeyId, null);
            }

            ValidationResult secretAccessKeyValidation = this.IsValidSecretAccessKey();
            if (!secretAccessKeyValidation.IsValid)
            {
                this.txtSecretAccessKey_Validating(txtSecretAccessKey, null);
            }

            result.Merge(secretAccessKeyValidation);
            return result;
        }
        
        /// <summary>
        /// Determines whether [is valid access key].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsValidAccessKey()
        {
            ValidationResult result = new ValidationResult(!this.txtAccessKeyId.Text.Trim().IsNullOrEmpty());
            if (!result.IsValid)
            {
                result.Messages.Add("Access Key ID is required");
            }
            
            return result;
        }

        /// <summary>
        /// Determines whether [is valid secret access key].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsValidSecretAccessKey()
        {
            ValidationResult result = new ValidationResult(!this.txtSecretAccessKey.Text.Trim().IsNullOrEmpty());
            if (!result.IsValid)
            {
                result.Messages.Add("Secret Access Key is required");
            }

            return result;    
        }       
    }
}
